<?php

namespace Drupal\c2pa_sign_aws_kms\Drush\Commands;

use Aws\Credentials\CredentialProvider;
use Aws\Kms\KmsClient;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A Drush commandfile.
 */
final class C2paSignAwsKmsCommands extends DrushCommands {

  /**
   * The config factory incase it is ever needed
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  private ConfigFactoryInterface $configFactory;

  /**
   * The config for this module
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $awsKmsconfig;

  /**
   * @var \Aws\Kms\KmsClient
   */
  private KmsClient $kmsClient;

  /**
   * Constructs a C2paSignAwsKmsCommands object.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
  ) {
    parent::__construct();
    $this->configFactory = $configFactory;
    $this->awsKmsconfig = $this->configFactory->get('c2pa_sign_aws_kms.settings');
    $this->kmsClient = c2pa_sign_aws_kms_create_client();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * Use AWS KMS to sign a message passed in through stdin, the signature will be provided through stdout.
   * #[CLI\Argument(name: 'arg1', description: 'Argument description.')]
   */
  #[CLI\Command(name: 'c2pa_sign_aws_kms:sign', aliases: ['c2pa_kms_sign'])]
  #[CLI\Option(name: 'alg', description: 'the signing algorithm')]
  #[CLI\Option(name: 'sign-cert', description: 'the certificate used to validate the signature')]
  #[CLI\Option(name: 'reserve-size', description: 'how much space is being reserved for the signature')]
  #[CLI\Usage(name: 'echo message.txt | c2pa_sign_aws_kms:command-name --alg es556 --reserve-size 20458 --sign-cert', description: 'Usage description')]
  public function c2pa_kms_sign($options = ['sign-cert' => NULL, 'alg' => 'es256', 'reserve-size' => NULL]) {
    $message_to_sign = file_get_contents('php://stdin');
    $settings = $this->awsKmsconfig->get('aws_kms');
    $digest = FALSE;
    // get the encryption algorithm
    # determine the aws kms signature based on what is passed in to the script;
    # KMS allowed signing algorithms
    #
    #          o RSASSA_PSS_SHA_256
    #          o RSASSA_PSS_SHA_384
    #          o RSASSA_PSS_SHA_512
    #          o RSASSA_PKCS1_V1_5_SHA_256
    #          o RSASSA_PKCS1_V1_5_SHA_384
    #          o RSASSA_PKCS1_V1_5_SHA_384
    #          o ECDSA_SHA_256
    #          o ECDSA_SHA_384
    #          o ECDSA_SHA_512
    switch ($options['alg']) {
      case 'es256':
        $algorithm = 'ECDSA_SHA_256';
        $digest_algorithm = 'sha256';
        break;
      case 'es384':
        $algorithm = 'ECDSA_SHA_384';
        $digest_algorithm = 'sha384';
        break;
      case 'es512':
        $algorithm = 'ECDSA_SHA_512';
        $digest_algorithm = 'sha512';
        break;
      case 'ps256':
        $algorithm = 'RSASSA_PKCS1_V1_5_SHA_256';
        $digest_algorithm = 'sha256';
        break;
      case 'ps384':
        $algorithm = 'RSASSA_PKCS1_V1_5_SHA_384';
        $digest_algorithm = 'sha384';
        break;
      case 'ps512':
        $algorithm = 'RSASSA_PKCS1_V1_5_SHA_512';
        $digest_algorithm = 'sha512';
        break;
      default;
        $this->logger()->error(dt('Signature algorithm @alg is currently not supported.', ['@alg' => $options['alg']]));
        return;
        break;
    }
    $digest = hash($digest_algorithm, $message_to_sign, TRUE);

    // request a signature
    $signatureResponse = $this->kmsClient->sign([
      'KeyId' => $settings['key'],
      'Message' => $digest,
      'MessageType' => 'DIGEST',
      'SigningAlgorithm' => $algorithm,
    ]);
    // get the signature from the response
    $signature = $signatureResponse->get('Signature');
    // output the signature
    echo $signature;
  }


}
